unit Main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls;

const
  st_Play = 0;
  st_GameOver = 1;

type
  TRow = array [0..14] of Boolean;

  TMainForm = class(TForm)
    tmrDown: TTimer;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure DrawPlayer;
    procedure EraseBackbuffer;
    procedure AddRow;
    procedure MoveDown;
    procedure tmrDownTimer(Sender: TObject);
    procedure DrawRows;
    procedure F_I_R_E;
    procedure AddBrick;
    procedure GameOver;
    procedure CheckRows;
    procedure RemoveRow(Y:ShortInt);
  private
    { Private declarations }
    bmpBrick:TBitmap;
    bmpBackground:TBitmap;
    bmpBackbuffer:TBitmap;
    plyX:ShortInt;
    Rows:array [0..18] of TRow;
    RowNum:ShortInt;
    State:Byte;
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.dfm}

procedure TMainForm.FormCreate(Sender: TObject);
begin
  bmpBrick := TBitmap.Create;
  bmpBrick.LoadFromFile( 'BRICK.bmp' );
  bmpBackground := TBitmap.Create;
  bmpBackground.LoadFromFile( 'Background.bmp' );
  bmpBackbuffer := TBitmap.Create;
  bmpBackbuffer.Width := ClientWidth;
  bmpBackbuffer.Height := ClientHeight;
  plyX := 6;
  RowNum := 0;
  AddRow;
  State := st_Play;
  tmrDown.Enabled := True;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  bmpBrick.Free;
  bmpBackground.Free;
  bmpBackbuffer.Free;
end;

procedure TMainForm.FormPaint(Sender: TObject);
begin
  EraseBackbuffer;
  DrawPlayer;
  DrawRows;
  Canvas.Draw( 0, 0, bmpBackbuffer);
end;

procedure TMainForm.FormKeyPress(Sender: TObject; var Key: Char);
var R:TRect;
begin
  if State = st_Play then
  begin
    if ('A' = UpCase(Key)) and (plyX >= 0) then Dec(plyX);
    if ('D' = UpCase(Key)) and (plyX <= 12) then Inc(plyX);
    if (' ' = UpCase(Key)) then F_I_R_E;
  end;
  if ( Key = #27 ) then Application.Terminate;
  SetRect( R, (plyX-1)*20, 18*20, (plyX+4)*20, 20*20 );
  InvalidateRect( Handle, @R, False );
end;

procedure TMainForm.DrawPlayer;
var C:TCanvas;
begin
  C := bmpBackbuffer.Canvas;
  C.Draw( plyX*20, 19*20, bmpBrick );
  C.Draw( (plyX+1)*20, 19*20, bmpBrick );
  C.Draw( (plyX+2)*20, 19*20, bmpBrick );
  C.Draw( (plyX+1)*20, 18*20, bmpBrick );
end;

procedure TMainForm.EraseBackbuffer;
begin
  bmpBackbuffer.Canvas.Draw( 0, 0, bmpBackground);
end;

procedure TMainForm.AddRow;
var X:ShortInt;
    R:TRect;
begin
  MoveDown;
  For X:=0 to 14 do
    Rows[0][X] := True;
  For X:= 0 to 5 do
    Rows[0][Random(15)] := False;
  SetRect( R, 0, 0, 15*20, RowNum*20 );
  InvalidateRect( Handle, @R, False );
  If RowNum = 19 then
  begin
    State := st_GameOver;
    GameOver;
  end;
end;

procedure TMainForm.MoveDown;
var Y:ShortInt;
begin
  For Y:= RowNum-1 downto 0 do
    Move( Rows[Y], Rows[Y+1], SizeOf(TRow) );
  Inc( RowNum );
end;

procedure TMainForm.tmrDownTimer(Sender: TObject);
begin
  AddRow;
  If tmrDown.Interval > 1500 then
    tmrDown.Interval := tmrDown.Interval - 10;
end;

procedure TMainForm.DrawRows;
var X,Y:ShortInt;
    C:TCanvas;
begin
  C := bmpBackbuffer.Canvas;
  For Y:=0 to RowNum-1 do
    For X:=0 to 14 do
      If Rows[Y][X] then C.Draw( X*20, Y*20, bmpBrick );
end;

procedure TMainForm.F_I_R_E;
var Y:ShortInt;
    R:TRect;
begin
  Y := RowNum -1;
  if RowNum = 0 then
    AddBrick
  else
    While (( Y >= 0 ) and ( Not Rows[Y][plyX+1] )) do
      Dec( Y );
    If Y = RowNum - 1 then
      AddBrick
    else
    begin
      Rows[Y+1][plyX+1] := True;
      SetRect( R, (plyX+1)*20, (Y+1)*20, (plyX+2)*20, (Y+2)*20 );
      InvalidateRect(Handle, Nil, false);
    end;
  CheckRows;
end;

procedure TMainForm.AddBrick;
var R:TRect;
begin
  FillChar( Rows[RowNum], SizeOf(TRow), 0 );
  Rows[RowNum][plyX+1] := True;
  Inc( RowNum );
  SetRect( R, 0, 0, 15*20, RowNum*20 );
  InvalidateRect( Handle, @R, False );
  If RowNum = 19 then
  begin
    State := st_GameOver;
    GameOver;
  end;
end;

procedure TMainForm.GameOver;
begin
  tmrDown.Enabled := False;
  tmrDown.Interval := 5000;
  if Application.MessageBox('Game Over! Hoces ponovo?','GameOver', MB_YESNO ) = ID_NO then
    Application.Terminate
  else
  begin
    plyX := 6;
    RowNum := 0;
    AddRow;
    State := st_Play;
    tmrDown.Enabled := True;
    InvalidateRect( Handle, Nil, False );
  end;
end;

procedure TMainForm.CheckRows;
var X, Y, C:ShortInt;
    Filled:Boolean;
    R:TRect;
begin
  C := RowNum-1;
  For Y := C downto 0 do
  begin
    Filled := True;
    For X := 0 to 14 do
    begin
      Filled := Filled and Rows[Y][X];
    end;
    If Filled then RemoveRow(Y);
  end;
  SetRect( R, 0, 0, 15*20, C*20 );
  InvalidateRect( Handle, @R, False );
end;

procedure TMainForm.RemoveRow(Y: ShortInt);
var I:ShortInt;
begin
  For I:=Y+1 to RowNum-1 do
    Move( Rows[I], Rows[I-1], SizeOf(TRow) );
  Dec(RowNum);
end;

end.
